@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# (c) Copyright 2003-2009 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2015 HP Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,28 +21,15 @@
 
 # StdLib
 import time
-import cStringIO
-
-from base.g import *
-try:
-   import xml.parsers.expat
-except ImportError,e:
-   log.info("\n")
-   log.error("Failed to import xml.parsers.expat(%s).\nThis may be due to the incompatible version of python-xml package.\n"%(e))
-   if "undefined symbol" in str(e):
-       log.info(log.blue("Please re-install compatible version (other than 2.7.2-7.14.1) due to bug reported at 'https://bugzilla.novell.com/show_bug.cgi?id=766778'."))
-       log.info(log.blue("\n        Run the following commands in root mode to change the python-xml package.(i.e Installing 2.7.2-7.1.2)"))
-       log.info(log.blue("\n        Using zypper:\n        'zypper remove python-xml'\n        'zypper install python-xml-2.7.2-7.1.2'"))
-       log.info(log.blue("\n        Using apt-get:\n        'apt-get remove python-xml'\n        'apt-get install python-xml-2.7.2-7.1.2'"))
-       log.info(log.blue("\n        Using yum:\n        'yum remove python-xml'\n        'yum install python-xml-2.7.2-7.1.2'"))
-
-   sys.exit(1)
-
+import io
+import binascii
+import xml.parsers.expat
 from string import *
 
 # Local
-from base.g import *
-from base import device, utils
+from .g import *
+from . import device, utils
+from .sixext import to_bytes_utf8
 
 http_result_pat = re.compile("""HTTP/\d.\d\s(\d+)""", re.I)
 HTTP_OK = 200
@@ -54,19 +41,53 @@ MAX_RETRIES = 2
 
 LEDM_WIFI_BASE_URI = "/IoMgmt/Adapters/"
 
-adapterPowerXml = """<io:Adapters xmlns:io=\"http://www.hp.com/schemas/imaging/con/ledm/iomgmt/2008/11/30\" xmlns:dd=\"http://www.hp.com/schemas/imaging/con/dictionaries/1.0/\"><io:Adapter><io:HardwareConfig><dd:Power>%s</dd:Power></io:HardwareConfig></io:Adapter></io:Adapters>"""
+# This payload is working for LaserJet Devices
+adapterPowerXml_payload2 ="""<?xml version="1.0" encoding="UTF-8" ?><io:Adapter xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:io="http://www.hp.com/schemas/imaging/con/ledm/iomgmt/2008/11/30" xmlns:dd="http://www.hp.com/schemas/imaging/con/dictionaries/1.0/" xmlns:wifi="http://www.hp.com/schemas/imaging/con/wifi/2009/06/26">  <io:HardwareConfig> <dd:Power>%s</dd:Power> </io:HardwareConfig> </io:Adapter>"""
+
+# This payload is working for OfficeJet and Photosmart Devices
+adapterPowerXml_payload1 = """<?xml version="1.0" encoding="UTF-8"?><io:Adapters xmlns:io="http://www.hp.com/schemas/imaging/con/ledm/iomgmt/2008/11/30" xmlns:dd="http://www.hp.com/schemas/imaging/con/dictionaries/1.0/"><io:Adapter><io:HardwareConfig><dd:Power>%s</dd:Power></io:HardwareConfig></io:Adapter></io:Adapters>"""
 
 passPhraseXml="""<io:Profile xmlns:io="http://www.hp.com/schemas/imaging/con/ledm/iomgmt/2008/11/30" xmlns:dd="http://www.hp.com/schemas/imaging/con/dictionaries/1.0/" xmlns:wifi="http://www.hp.com/schemas/imaging/con/wifi/2009/06/26" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.hp.com/schemas/imaging/con/ledm/iomgmt/2008/11/30 ../../schemas/IoMgmt.xsd http://www.hp.com/schemas/imaging/con/dictionaries/1.0/ ../../schemas/dd/DataDictionaryMasterLEDM.xsd"><io:AdapterProfile><io:WifiProfile><wifi:SSID>%s</wifi:SSID><wifi:CommunicationMode>%s</wifi:CommunicationMode><wifi:EncryptionType>%s</wifi:EncryptionType><wifi:AuthenticationMode>%s</wifi:AuthenticationMode></io:WifiProfile></io:AdapterProfile></io:Profile>"""
 
 keyInfoXml = """<io:KeyInfo><io:WpaPassPhraseInfo><wifi:RsnEncryption>AESOrTKIP</wifi:RsnEncryption><wifi:RsnAuthorization>autoWPA</wifi:RsnAuthorization><wifi:PassPhrase>%s</wifi:PassPhrase></io:WpaPassPhraseInfo></io:KeyInfo>"""
 
+def flushThePort(dev):
+    response = io.BytesIO()
+    timeout = 1
+    if dev.openLEDM() == -1:
+        dev.closeLEDM()
+        if dev.openEWS_LEDM() == -1:            
+            dev.openMarvell_EWS()
+            try:
+                while dev.readMarvell_EWS(1024, response, timeout):
+                    pass
+            except Error:                
+                log.error("Unable to read Marvell_EWS Channel")
+            finally:
+                dev.closeMarvell_EWS() 
+        else:
+            try:
+                dev.readLEDMAllData(dev.readEWS_LEDM, response, timeout)
+            except Error:                
+                log.error("Unable to read EWS_LEDM Channel")
+            finally:
+                dev.closeEWS_LEDM()    
+    else:
+        try:
+            dev.readLEDMAllData(dev.readLEDM, response, timeout)
+        except Error:            
+            log.error("Unable to read LEDM Channel")
+        finally:
+            dev.closeLEDM()
+
 def getAdaptorList(dev):
     ret,params,elementCount,code ={},{},0,HTTP_ERROR         
     max_tries = 0
+    flushThePort(dev)
     while max_tries < MAX_RETRIES:
         max_tries +=1
         URI = LEDM_WIFI_BASE_URI[0:len(LEDM_WIFI_BASE_URI)-1]# to remove "\" from the string
-        params,code,elementCount = readXmlDataFromURI(dev,URI,'<io:Adapters', '<io:Adapter>')
+        paramsList,code = readXmlTagDataFromURI(dev,URI,'<io:Adapters', '<io:Adapter>')
         if code == HTTP_OK:
             break
 
@@ -74,32 +95,34 @@ def getAdaptorList(dev):
         log.error("Request Failed With Response Code %d"%code)
         return ret
 
-    ret['adaptorlistlength'] = elementCount
-    if params is not None:        
-        if elementCount == 1:
-            try:
-                ret['adaptorid-0' % a] = params['io:adapters-io:adapter-map:resourcenode-map:resourcelink-dd:resourceuri']
-                ret['adaptorname-0' % a] = params['io:adapters-io:adapter-io:hardwareconfig-dd:name']
-                ret['adaptorpresence-0' % a] = ''
-                ret['adaptorstate-0' % a] = ''
-                ret['adaptortype-0' % a] = params['io:adapters-io:adapter-io:hardwareconfig-dd:deviceconnectivityporttype']                
-            except KeyError, e:
-                log.debug("Missing response key: %s" % e)   # changed from error to debug
-        else:
-            for a in xrange(elementCount):
+    ret['adaptorlistlength'] = len(paramsList)
+    if len(paramsList) != 0:        
+            a = 0
+            for params in paramsList:
+                ret['adaptorpresence-%d' % a] = ''
+                ret['adaptorstate-%d' % a] = ''
                 try:
-                    ret['adaptorid-%d' % a] = params['io:adapters-io:adapter-map:resourcenode-map:resourcelink-dd:resourceuri-%d' % a]
-                    ret['adaptorname-%d' % a] = params['io:adapters-io:adapter-io:hardwareconfig-dd:name-%d' % a]
-            	    ret['adaptorpresence-%d' % a] = ''
-            	    ret['adaptorstate-%d' % a] = ''
-            	    ret['adaptortype-%d' % a] = params['io:adapters-io:adapter-io:hardwareconfig-dd:deviceconnectivityporttype-%d' % a]            	    
-                except KeyError, e:
+                    ret['adaptorid-%d' % a] = params['io:adapter-map:resourcenode-map:resourcelink-dd:resourceuri']
+                except KeyError as e:
                     log.debug("Missing response key: %s" % e)    #changed from error to debug
+                    ret['adaptorid-%d' % a]=""
+                try:
+                    ret['adaptorname-%d' % a] = params['io:adapter-io:hardwareconfig-dd:name']
+                except KeyError as e:
+                    log.debug("Missing response key: %s" % e)    #changed from error to debug
+                    ret['adaptorname-%d' % a] = ""
+                try:
+                    ret['adaptortype-%d' % a] = params['io:adapter-io:hardwareconfig-dd:deviceconnectivityporttype']
+                except KeyError as e:
+                    log.debug("Missing response key: %s" % e)    #changed from error to debug
+                    ret['adaptortype-%d' % a] = ""
+                    
+                a = a+1
     return ret   
 
 
 def getWifiAdaptorID(dev):
-    ret = {}
+    rVal = []
 
     ret = getAdaptorList(dev)
     try:
@@ -107,7 +130,7 @@ def getWifiAdaptorID(dev):
     except KeyError:
         num_adaptors = 0
 
-    for n in xrange(num_adaptors):
+    for n in range(num_adaptors):
         try:
             name = ret['adaptortype-%d' % n]
         except KeyError:
@@ -127,20 +150,34 @@ def getWifiAdaptorID(dev):
 
                 r.append(x)
 
-            return r
+            rVal.append(r)
 
-    return -1, 'Unknown', 'Unknown', 'Unknown'
+    return rVal
                          
-def setAdaptorPower(dev, adapterName, adaptor_id=0, power_state='on'):
-    ret,powerXml,URI,code = {},'','',HTTP_ERROR
-    URI = LEDM_WIFI_BASE_URI + adapterName
-    powerXml = adapterPowerXml %(power_state)  
+def setAdaptorPower(dev, adapterList, power_state='on'):
+    adaptor_id=-1
+    adaptorName =""
+    for a in adapterList:
+       adaptor_id = a[0]
+       adaptorName = a[1]
+       ret,powerXml,URI,code = {},'','',HTTP_ERROR
+       URI = LEDM_WIFI_BASE_URI + adaptorName
+       powerXml = adapterPowerXml_payload1 %(power_state)  
   
-    ret['errorreturn'] = writeXmlDataToURI(dev,URI,powerXml,10)    
-    if not(ret['errorreturn'] == HTTP_OK or HTTP_NOCONTENT):
-        log.error("Request Failed With Response Code %d" %code)
-    
-    return ret
+       ret['errorreturn'] = writeXmlDataToURI(dev,URI,powerXml,60)
+       if not(ret['errorreturn'] == HTTP_OK or ret['errorreturn'] == HTTP_NOCONTENT):
+          log.debug("Wifi Adapter turn ON request Failed. ResponseCode=%s AdaptorId=%s AdaptorName=%s. Trying another interface" %(ret['errorreturn'],adaptor_id,adaptorName))
+          powerXml = adapterPowerXml_payload2 %(power_state)
+          ret['errorreturn'] = writeXmlDataToURI(dev,URI,powerXml,15)
+
+       if not(ret['errorreturn'] == HTTP_OK or ret['errorreturn'] == HTTP_NOCONTENT):
+          log.error("Wifi Adapter turn ON request Failed. ResponseCode=%s AdaptorId=%s AdaptorName=%s" %(ret['errorreturn'],adaptor_id,adaptorName))
+       else:
+          log.debug("Wifi Adapter turn ON request is Success. AdaptorId=%s AdaptorName=%s" %(adaptor_id,adaptorName))
+#          adapaterState = a[2], adapterPresence= a[3] 
+          return adaptor_id, adaptorName, a[2], a[3] 
+
+    return -1 ,"","",""
 
 def performScan(dev, adapterName, ssid=None):
     ret ={}
@@ -151,7 +188,7 @@ def performScan(dev, adapterName, ssid=None):
         URI = LEDM_WIFI_BASE_URI + adapterName + "/WifiNetworks/SSID="+ssid 
 
     while True:            
-        params,code,elementCount = readXmlDataFromURI(dev,URI,'<io:WifiNetworks', '<io:WifiNetwork>',10)        
+        params,code,elementCount = readXmlDataFromURI(dev,URI,'<io:WifiNetworks', '<io:WifiNetwork>',30)        
         if code == HTTP_ACCEPTED:
             continue
         else:
@@ -165,13 +202,13 @@ def performScan(dev, adapterName, ssid=None):
     if params is not None:              
         if elementCount == 1:
             try:
-                ssid = str(params['io:wifinetworks-io:wifinetwork-wifi:ssid']).decode("hex")
+                ssid = binascii.unhexlify(str(params['io:wifinetworks-io:wifinetwork-wifi:ssid']).encode('utf-8')).decode("utf-8")
                 if not ssid:
-                    ret['ssid-0'] = u'(unknown)'
+                    ret['ssid-0'] = to_unicode('(unknown)')
                 else:
                     ret['ssid-0'] = ssid
                 try:
-                    ret['bssid-0'] = str(params['io:wifinetworks-io:wifinetwork-wifi:bssid']).decode("hex")
+                    ret['bssid-0'] = binascii.unhexlify(str(params['io:wifinetworks-io:wifinetwork-wifi:bssid']).encode('utf-8')).decode("utf-8")
                 except:
                     ret['bssid-0'] = params['io:wifinetworks-io:wifinetwork-wifi:bssid']
                                    
@@ -180,37 +217,39 @@ def performScan(dev, adapterName, ssid=None):
                 ret['dbm-0'] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:dbm']
                 ret['encryptiontype-0'] = params['io:wifinetworks-io:wifinetwork-wifi:encryptiontype']
                 ret['signalstrength-0'] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:signalstrength']                
-            except KeyError, e:
-                log.error("Missing response key: %s" % e)  
+            except KeyError as e:
+                log.debug("Missing response key: %s" % e)  
         else:
-            for a in xrange(elementCount):
-                try:                
+            for a in range(elementCount):
+                try:
                     try:
-                        ssid = str(params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]).decode("hex")
-                        #ssid = params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]
-                    except:
+                        ssid = binascii.unhexlify(str(params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]).encode('utf-8')).decode('utf-8')
+                    except TypeError: 
+                        # Some devices returns one invalid SSID (i.e. 0) along with valid SSIDs. e.g. Epic.
                         ssid = params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]
-                        #ssid = str(params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]).decode("hex")
+                    except binascii.Error as err:
+                        ssid = params['io:wifinetworks-io:wifinetwork-wifi:ssid-%d' % a]
+
                     if not ssid:
-                        ret['ssid-%d' % a] = u'(unknown)'
+                        ret['ssid-%d' % a] = to_unicode('(unknown)')
                     else:
                         ret['ssid-%d' % a] = ssid
                     try:
-                        ret['bssid-%d' % a] = str(params['io:wifinetworks-io:wifinetwork-wifi:bssid-%d' % a]).decode("hex")
+                        ret['bssid-%d' % a] = binascii.unhexlify(str(params['io:wifinetworks-io:wifinetwork-wifi:bssid-%d' % a]).encode('utf-8')).decode("utf-8")
                     except:
                         ret['bssid-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:bssid-%d' % a]
-            	    ret['channel-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:channel-%d' % a]
-            	    ret['communicationmode-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:communicationmode-%d' % a]
-            	    ret['dbm-%d' % a] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:dbm-%d' % a]
-            	    ret['encryptiontype-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:encryptiontype-%d' % a]
-                    ret['signalstrength-%d' % a] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:signalstrength-%d' % a]            	    
-            	
-                except KeyError, e:
-                    log.error("Missing response key: %s" % e)  
+                    ret['channel-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:channel-%d' % a]
+                    ret['communicationmode-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:communicationmode-%d' % a]
+                    ret['dbm-%d' % a] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:dbm-%d' % a]
+                    ret['encryptiontype-%d' % a] = params['io:wifinetworks-io:wifinetwork-wifi:encryptiontype-%d' % a]
+                    ret['signalstrength-%d' % a] = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:signalstrength-%d' % a]                        
+                
+                except KeyError as e:
+                    log.debug("Missing response key: %s" % e)  
                 try:                    
                     ret['signalstrengthmax'] = 5
                     ret['signalstrengthmin'] = 0
-                except KeyError, e:
+                except KeyError as e:
                     log.debug("Missing response key: %s" % e)       
     return ret    
 
@@ -256,12 +295,12 @@ def getIPConfiguration(dev, adapterName):
                     if elementCount ==1:
                         pridns = params['io:protocols-io:protocol-dd:dnsserveripaddress']
                         sec_dns = params['io:protocols-io:protocol-dd:secondarydnsserveripaddress']          
-                        for a in xrange(elementCount):
+                        for a in range(elementCount):
                             if params['io:protocols-io:protocol-dd:dnsserveripaddress-%d' %a] !="::":
                                 pridns = params['io:protocols-io:protocol-dd:dnsserveripaddress-%d' %a]
                                 sec_dns = params['io:protocols-io:protocol-dd:secondarydnsserveripaddress-%d' %a]
                                 break
-            except KeyError, e:
+            except KeyError as e:
                 log.error("Missing response key: %s" % str(e))
     else:
         if params is not None and code == HTTP_OK:
@@ -271,16 +310,30 @@ def getIPConfiguration(dev, adapterName):
                     ip = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:ipaddress']
                 except:
                     ip = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:ipaddress-0']
-            #subnetmask = params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:subnetmask']
+
+                #subnetmask = params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:subnetmask']
+                try:
                     subnetmask = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:subnetmask']
-            #gateway = params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:defaultgateway']
+                except:
+                    subnetmask = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:subnetmask-0']
+
+                #gateway = params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:defaultgateway']
+                try:
                     gateway = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:defaultgateway']
-            
-            #if 'DHCP' in params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:configmethod']:
-                    if 'DHCP' in params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:ipconfigmethod-0']:
-                        addressmode = 'dhcp'
-                    else:
-                        addressmode = 'autoip'    
+                except:
+                    gateway = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:defaultgateway-0']
+
+                #if 'DHCP' in params['io:protocols-io:protocol-io:addresses-io:ipv4addresses-io:ipv4address-dd:configmethod']:
+                try:
+                    addressmode = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:ipconfigmethod'] 
+                except:
+                    addressmode = params['iocfgdyn2:ioconfigdyn-dd3:ioadaptorconfig-dd3:networkadaptorconfig-dd3:ipversionconfig-dd3:ipconfig-dd:ipconfigmethod-0'] 
+
+                if 'dhcp' in addressmode.lower():
+                    addressmode = 'dhcp'
+                else:
+                    addressmode = 'autoip'
+
             #if elementCount ==1:
             #    pridns = params['io:protocols-io:protocol-dd:dnsserveripaddress']
             #    sec_dns = params['io:protocols-io:protocol-dd:secondarydnsserveripaddress']          
@@ -289,12 +342,47 @@ def getIPConfiguration(dev, adapterName):
             #        pridns = params['io:protocols-io:protocol-dd:dnsserveripaddress-%d' %a]
             #        sec_dns = params['io:protocols-io:protocol-dd:secondarydnsserveripaddress-%d' %a]
             #        break
-            except KeyError, e:
+            except KeyError as e:
                 log.error("Missing response key: %s" % str(e))        
 
+    log.debug("ip=%s, hostname=%s, addressmode=%s, subnetmask=%s, gateway=%s, pridns=%s, sec_dns=%s"%(ip, hostname, addressmode, subnetmask, gateway, pridns, sec_dns))
     return ip, hostname, addressmode, subnetmask, gateway, pridns, sec_dns  
 
 
+
+
+# TODO: Temporary Function. To be removed after refactoring.
+def getwifiotherdetails(dev,adapterName):
+    ip, subnet, gateway, pri_dns, sec_dns, mode = '', '', '', '', '', ''
+    params1, params2, code1, code2, elementCount ={}, {}, HTTP_ERROR, HTTP_ERROR,0
+    URI1 = LEDM_WIFI_BASE_URI + adapterName + "/Profiles/Active"
+    URI2 = "/IoMgmt/IoConfig.xml" 
+    max_tries = 0
+
+    while max_tries < MAX_RETRIES:
+        max_tries +=1
+        params1, code1, elementCount = readXmlDataFromURI(dev,URI1,'<io:Profile', '<io:Profile')
+        params2, code2, elementCount = readXmlDataFromURI(dev,URI2,'<io:IoConfig', '<io:IoConfig')
+        if code1 == HTTP_OK and code2 == HTTP_OK:
+            break 
+
+    if code1 !=HTTP_OK and code2 != HTTP_OK:
+        log.error("Request Failed With Response Code %d" %code)
+        return ip, subnet, gateway, pri_dns, sec_dns 
+	
+    if params1 is not None and params2 is not None:
+        try:
+            ip = params1['io:profile-io:networkprofile-io:ipv4network-dd:ipaddress']
+            subnet = params1['io:profile-io:networkprofile-io:ipv4network-dd:subnetmask']
+            gateway = params1['io:profile-io:networkprofile-io:ipv4network-dd:defaultgateway']
+            pri_dns  = params1['io:profile-io:networkprofile-io:ipv4network-dd:dnsserveripaddress']
+            sec_dns = params1['io:profile-io:networkprofile-io:ipv4network-dd:secondarydnsserveripaddress']
+            mode = params2['io:ioconfig-io:iodeviceprotocolconfig-io:ipv4domainname-dd:domainnameconfig-dd:configmethod']
+
+        except KeyError as e:
+            log.debug("Missing response key: %s" % str(e))
+    return ip, subnet, gateway, pri_dns, sec_dns, mode
+	
 def getCryptoSuite(dev, adapterName):
     alg, mode, secretid = '', '', ''
     parms,code,elementCount ={},HTTP_ERROR,0
@@ -316,8 +404,8 @@ def getCryptoSuite(dev, adapterName):
             mode = parms['io:profile-io:adapterprofile-io:wifiprofile-wifi:communicationmode']
             alg = parms['io:profile-io:adapterprofile-io:wifiprofile-wifi:encryptiontype']
             secretid = parms['io:profile-io:adapterprofile-io:wifiprofile-wifi:bssid']    
-        except KeyError, e:
-            log.error("Missing response key: %s" % str(e))
+        except KeyError as e:
+            log.debug("Missing response key: %s" % str(e))
     
     return  alg, mode, secretid
 
@@ -328,14 +416,14 @@ def associate(dev, adapterName, ssid, communication_mode, encryption_type, key):
 
     if encryption_type == 'none':
         authMode = 'open'
-        ppXml = passPhraseXml%(ssid.encode('hex'),communication_mode,encryption_type,authMode)
+        ppXml = passPhraseXml%(binascii.hexlify(to_bytes_utf8(ssid)).decode('utf-8'), communication_mode,encryption_type,authMode)
     else:
         authMode = encryption_type
         pos = passPhraseXml.find("</io:WifiProfile>",0,len(passPhraseXml))
-        ppXml = (passPhraseXml[:pos] + keyInfoXml + passPhraseXml[pos:])%(ssid.encode('hex'),communication_mode,encryption_type,\
-        authMode,key.encode('hex'))        
+        ppXml = (passPhraseXml[:pos] + keyInfoXml + passPhraseXml[pos:])%(binascii.hexlify(to_bytes_utf8(ssid)).decode('utf-8'),communication_mode,encryption_type,\
+        authMode,binascii.hexlify(to_bytes_utf8(key)).decode('utf-8'))        
 
-    code = writeXmlDataToURI(dev,URI,ppXml,10)    
+    code = writeXmlDataToURI(dev,URI,ppXml,15)
     ret['errorreturn'] = code
     if not(code == HTTP_OK or HTTP_NOCONTENT):
         log.error("Request Failed With Response Code %d" % ret['errorreturn'])
@@ -356,7 +444,7 @@ def getVSACodes(dev, adapterName):
             break
     
     if code != HTTP_OK:
-        log.error("Request Failed With Response Code %d"%code)
+        log.warn("Request Failed With Response Code %d"%code)
         return ret
  
     if params is not None:
@@ -376,7 +464,7 @@ def getVSACodes(dev, adapterName):
 
 def getHostname(dev):
     hostName = ''
-    URI = "/IoMgmt/IoConfig.xml"
+    URI = "/IoMgmt/IoConfig.xml"    
     max_tries = 0
     
     while max_tries < MAX_RETRIES:
@@ -386,14 +474,14 @@ def getHostname(dev):
             break    
     
     if code != HTTP_OK:
-        log.error("Request failed with Response code %d"%code)
-        return hostName
+        log.warn("Request failed with Response code %d. HostName not found."%code)
+        return  hostName
        
     if params is not None:        
         try:               
             hostName = params['io:ioconfig-io:iodeviceconfig-dd3:hostname']            
-        except KeyError, e:
-            log.error("Missing response key: %s" % e)
+        except KeyError as e:
+            log.debug("Missing response key: %s" % e)
 
     return  hostName
 
@@ -423,51 +511,123 @@ def getSignalStrength(dev, adapterName, ssid, adaptor_id=0):
             try:                
                 ss_dbm = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:dbm']                
                 ss_val = params['io:wifinetworks-io:wifinetwork-io:signalinfo-wifi:signalstrength']                
-            except KeyError, e:
+            except KeyError as e:
                 log.error("Missing response key: %s" % e)
 
     return  ss_max, ss_min, ss_val, ss_dbm
 
 
+def readXmlTagDataFromURI(dev,URI,xmlRootNode,xmlReqDataNode,timeout=5):
+    paramsList,code =[],HTTP_ERROR
+    
+    data = format_http_get(URI,0,"")
+    log.info(data)
+    response = io.BytesIO()
+    if dev.openLEDM() == -1:
+        dev.closeLEDM()
+        if dev.openEWS_LEDM() == -1:
+            
+            dev.openMarvell_EWS()
+            dev.writeMarvell_EWS(data)
+            try:
+                while dev.readMarvell_EWS(1024, response, timeout):
+                    pass
+            except Error:
+                dev.closeMarvell_EWS()
+                log.error("Unable to read Marvell_EWS Channel")
+        else:
+            dev.writeEWS_LEDM(data)
+            try:
+                dev.readLEDMData(dev.readEWS_LEDM, response, timeout)
+            except Error:
+                dev.closeEWS_LEDM()
+                log.error("Unable to read EWS_LEDM Channel")
+    else:
+        dev.writeLEDM(data)
+        try:
+            dev.readLEDMData(dev.readLEDM, response, timeout)
+        except Error:
+            dev.closeLEDM()
+            log.error("Unable to read LEDM Channel")
+        
+    strResp = response.getvalue().decode('utf-8')
+    if strResp is not None:                             
+        code = get_error_code(strResp)
+        if code == HTTP_OK:
+            #strResp = utils.unchunck_xml_data(strResp)
+            strResp = utils.extract_xml_chunk(strResp)
+            pos = strResp.find(xmlRootNode,0,len(strResp))    
+            repstr = strResp[pos:].strip()
+            repstr = repstr.replace('\r',' ').replace('\t',' ').replace('\n',' ') # To remove formating characters from the received xml
+            repstr = repstr.rstrip('0')   # To remove trailing zero from the received xml
+            try:
+                parser_object = utils.extendedExpat()
+                root_element = parser_object.Parse(repstr)
+                xmlReqDataNode = ''.join(l for l in filter(lambda x: x not in '<>', xmlReqDataNode)) # [c for c in xmlReqDataNode if c not in "<>"] # To remove '<' and '>' characters
+                reqDataElementList = root_element.getElementsByTagName(xmlReqDataNode)
+                for node in reqDataElementList:
+                    repstr = node.toString()
+                    repstr = repstr.replace('\r',' ').replace('\t',' ').replace('\n',' ') # To remove formating characters from the received xml
+                    params = utils.XMLToDictParser().parseXML(to_bytes_utf8(repstr))
+                    paramsList.append(params)
+            except xml.parsers.expat.ExpatError as e:
+                log.debug("XML parser failed: %s" % e)  #changed from error to debug 
+        else:
+            log.debug("HTTP Responce failed with %s code"%code)
+    return paramsList,code
+
+
+
 def readXmlDataFromURI(dev,URI,xmlRootNode,xmlChildNode,timeout=5):
     params,code,elementCount ={},HTTP_ERROR,0 
     
     data = format_http_get(URI,0,"")
     log.info(data)
-    response = cStringIO.StringIO()
+    response = io.BytesIO()
     if dev.openLEDM() == -1:
-        dev.openEWS_LEDM()
-        dev.writeEWS_LEDM(data)
-        try:
-            while dev.readEWS_LEDM(1024, response, timeout):
-                pass
-        except Error:
-            dev.closeEWS_LEDM()
-            log.error("Unable to read EWS_LEDM Channel")
+        dev.closeLEDM()
+        if dev.openEWS_LEDM() == -1:
+            
+            dev.openMarvell_EWS()
+            dev.writeMarvell_EWS(data)
+            try:
+                while dev.readMarvell_EWS(1024, response, timeout):
+                    pass
+            except Error:
+                dev.closeMarvell_EWS()
+                log.error("Unable to read Marvell_EWS Channel")
+        else:
+            dev.writeEWS_LEDM(data)
+            try:
+                dev.readLEDMData(dev.readEWS_LEDM, response,timeout)
+            except Error:
+                dev.closeEWS_LEDM()
+                log.error("Unable to read EWS_LEDM Channel")
     else:
         dev.writeLEDM(data)
-
-    #response = cStringIO.StringIO()
         try:
-            while dev.readLEDM(1024, response, timeout):
-                pass
+            dev.readLEDMData(dev.readLEDM, response,timeout)
         except Error:
             dev.closeLEDM()
             log.error("Unable to read LEDM Channel") 
     #dev.closeEWS_LEDM()    
-    strResp = str(response.getvalue())
-    #log.error(strResp)
-    if strResp is not None:                         	
-        code = get_error_code(strResp)        
-    	pos = strResp.find(xmlRootNode,0,len(strResp))    
-    	repstr = strResp[pos:].strip()
-        repstr = filter(lambda c: c not in "\r\t\n", repstr) # To remove formating characters from the received xml
-        repstr = repstr.rstrip('0')   # To remove trailing zero from the received xml
-    	elementCount = repstr.count(xmlChildNode)          	    	
-    	try:
-            params = utils.XMLToDictParser().parseXML(repstr)            
-        except xml.parsers.expat.ExpatError, e:
-            log.debug("XML parser failed: %s" % e)  #changed from error to debug 
+    strResp = response.getvalue().decode('utf-8')
+    if strResp is not None:                             
+        code = get_error_code(strResp)
+        if code == HTTP_OK:
+            #strResp = utils.unchunck_xml_data(strResp)
+            strResp = utils.extract_xml_chunk(strResp)
+            pos = strResp.find(xmlRootNode,0,len(strResp))    
+            repstr = strResp[pos:].strip()
+            repstr = repstr.replace('\r',' ').replace('\t',' ').replace('\n',' ') # To remove formating characters from the received xml
+            repstr = repstr.rstrip('0')   # To remove trailing zero from the received xml
+            elementCount = repstr.count(xmlChildNode)
+            try:
+                params = utils.XMLToDictParser().parseXML(repstr)            
+            except xml.parsers.expat.ExpatError as e:
+                log.debug("XML parser failed: %s" % e)  #changed from error to debug 
+        else:
+            log.debug(" HTTP Responce failed with %s code"%code)
 
     return params,code,elementCount
 
@@ -476,30 +636,36 @@ def writeXmlDataToURI(dev,URI,xml,timeout=5):
     code = HTTP_ERROR
 
     data = format_http_put(URI,len(xml),xml)
-    response = cStringIO.StringIO()
+    response = io.BytesIO()
 
     if dev.openLEDM() == -1:
-        dev.openEWS_LEDM()
-        dev.writeEWS_LEDM(data)
-        try:
-            while dev.readEWS_LEDM(1000, response, timeout):
-                pass
-        except Error:
+        if dev.openEWS_LEDM() == -1:
+            dev.openMarvell_EWS()
+            dev.writeMarvell_EWS(data)
+            try:
+               while dev.readMarvell_EWS(1000, response, timeout):
+                   pass
+            except Error:
+                dev.closeMarvell_EWS()
+                log.error("Unable to read Marvell_EWS Channel")
+        else:
+            dev.writeEWS_LEDM(data)
+            try:
+                dev.readLEDMData(dev.readEWS_LEDM, response, timeout)
+            except Error:
                 dev.closeEWS_LEDM()
                 log.error("Unable to read EWS_LEDM Channel")
             
     else:
         dev.writeLEDM(data)
-    #response = cStringIO.StringIO()
         try:
-            while dev.readLEDM(1000, response, timeout):
-                pass
+            dev.readLEDMData(dev.readLEDM, response,timeout )
         except Error:
             dev.closeLEDM()
             log.error("Unable to read LEDM Channel") 
         
 
-    strResp = str(response.getvalue())    
+    strResp = response.getvalue().decode('utf-8')
     if strResp is not None:
         code = get_error_code(strResp)           
     return code
@@ -538,4 +704,3 @@ Content-Type: $content_type\r
 Content-Length: $ledmlen\r
 \r
 $xmldata""")    
-    
